//
//  Copyright (c) 2012 Artyom Beilis (Tonkikh)
//
//  Distributed under the Boost Software License, Version 1.0. (See
//  accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
#ifndef NOWIDE_CSTDLIB_HPP_INCLUDED
#define NOWIDE_CSTDLIB_HPP_INCLUDED

#include <nowide/cenv.hpp>
#include <nowide/system.hpp>

#endif
///
// vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
