
/******************************************************************************
* MODULE     : aqua_dialogues.hpp
* DESCRIPTION: Aqua dialogues widgets
* COPYRIGHT  : (C) 2007  Massimiliano Gubinelli
*******************************************************************************
* This software falls under the GNU general public license version 3 or later.
* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
******************************************************************************/

#ifndef AQUA_DIALOGUES_H
#define AQUA_DIALOGUES_H

#include "aqua_widget.h"

#endif // defined AQUA_DIALOGUES_H
