// Resource file for TeXmacs' main executable.
// Compile with windres resource.rc -O coff -o resource.rc.o and link as any
// other object file

appicon ICON "TeXmacs.ico"

#include <winver.h>

// Get updates from this appcast feed:
FeedURL   APPCAST   {""}

// Version information; this, too, is used by WinSparkle
1 VERSIONINFO
 FILEVERSION     2,1,1,0
 PRODUCTVERSION  2,1,1,0
 FILEFLAGSMASK   VS_FFI_FILEFLAGSMASK
 FILEFLAGS       (VS_FF_PATCHED|0)
 FILEOS          VOS_NT_WINDOWS32
 FILETYPE        VFT_APP
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904B0" // US English, Unicode
    BEGIN
      VALUE "Comments", "TeXmacs, a scientific edition platform."
      VALUE "CompanyName", "texmacs.org\0"
      VALUE "FileDescription", "TeXmacsQt Windows\0"
      VALUE "FileVersion", "2.1.1\0"
      VALUE "InternalName", "texmacs\0"
      VALUE "OriginalFilename", "texmacs.exe\0"
      VALUE "LegalCopyright", "Licensed under GPLv3\0"
      VALUE "ProductName", "TeXmacs\0"
      VALUE "ProductVersion", "2.1.1\0"
    END
  END

  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x0409, 0x04B0
  END
END
